#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import os, string, sys
import math
from array import array

#get pixel helper function
def get_pixel(point,image_width,p_size,src_pixels) :
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	pix = src_pixels[src_pos: src_pos + p_size]
	if p_size == 3:    #if it doesn't have alpha information add full opacity
		pix.append(255)
	return (pix)
def set_pixel(point,image_width,p_size,src_pixels,color):
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	#src_pixels[src_pos: src_pos + p_size] = array("B",color).tostring()
	src_pixels[src_pos] = color[0]
	src_pixels[src_pos+1] = color[1]
	src_pixels[src_pos+2] = color[2]
	src_pixels[src_pos+3] = color[3]
	return None
	
#helper function to convert a list of bytes to color tuple
def color(pixeldata):
	return ((pixeldata[0],pixeldata[1],pixeldata[2]))
	
#returns true if pixel is pure black
def pureblack(pixel):
	if pixel[0] == 0 and pixel[1] == 0 and pixel[2] == 0 and pixel[3] == 255:
		return True
	else:
		return False
def gray_from_height(height):
	return ((height,height,height))
	
def pointed_raster(image,layer,ok):
	pdb.gimp_layer_resize_to_image_size(layer) # make layer same as image size to not deal with possible layer offsets
	segments = int(layer.width)-1 #make gradient as large as width
	#grab layer's pixel information
	srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height, True, False)
	p_size = len(srcRgn[0,0])
	
	#dest_pixels = array("B", "\x00" * (newWidth * newHeight * p_size))
	src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height])
	
	
	#pdb.gimp_message(str(src_pixels))
	#pdb.gimp_message("width:" + str(layer.width))
	#pdb.gimp_message("height:" + str(layer.height))
	
	#detecting from left side
	
	selection = pdb.gimp_selection_bounds(image)
	selected = selection[0]
	if selected < 1:
		pdb.gimp_message("You need to make a selection!")
		raise Exception("You need to make a selection!")
		
		
	x1 = selection[1]
	y1 = selection[2]
	x2 = selection[3]
	y2 = selection[4]
	
	#pdb.gimp_message(str(selection))
	left_ = [layer.width-1] * layer.height
	gimp.progress_init("Detecting left pixels...")
	for y in range(y1,y2+1):
		gimp.progress_update(1.0 * y/layer.height)
		for x in range(x1,x2+1):
			frac = pdb.gimp_selection_value(image,x,y)
			if pdb.gimp_selection_value(image,x,y) > 0: #is in selection
				left_[y] = x
				set_pixel((x,y),layer.width,p_size,src_pixels,[255, 0, 0, 255])
				break
    #detecting from right side
	right_ = [0] * layer.height
	gimp.progress_init("Detecting right pixels...")
	for y in range(y1,y2+1):
		gimp.progress_update(1.0 * y/layer.height)
		for x in range(x2,x1-1,-1):
			if pdb.gimp_selection_value(image,x,y) > 0: #is in selection
				right_[y] = x
				set_pixel((x,y),layer.width,p_size,src_pixels,[0, 0, 255, 255])
				break
				
	radius_ = [0] * layer.height
	midpoint_ = [0] * layer.height
	gimp.progress_init("Calculating radius, midpoints...")
	
	for y in range(0,layer.height):
		gimp.progress_update(1.0 * y/layer.height)
		diameter = right_[y] -  left_[y]
		radius_[y] = 1.0 * diameter / 2.0
		midpoint_[y] = (right_[y] + left_[y]) / 2.0

	max_radius = max(radius_)
	gimp.progress_init("Rendering...")
	for y in range(y1,y2+1):
		gimp.progress_update(1.0*(y-y1)/(((y2+1))-y1))
		for x in range(left_[y],right_[y]+1):
			base = abs(x - midpoint_[y])
			height = radius_[y]-base
			
			c_value = 255 - int((height/max_radius) * 255.0)
			set_pixel((x,y),layer.width,p_size,src_pixels,[c_value, c_value, c_value, 255])
	
	
	srcRgn[0:layer.width, 0:layer.height] = src_pixels.tostring()
	layer.flush()
	layer.update(0,0,layer.width,layer.height)
	
	
	pdb.gimp_message("Done!")	
	
	
register(
    "python_fu_pointed_raster_from_selection",
    "Creates Pointed Raster height map for PegLeg44 from selection",
    "Creates Pointed Raster height map for PegLeg44 from selection",
    "Tin Tran",
    "GPL",
    "May 2016",
    "<Image>/Python-Fu/Pointed Raster Experiment...",
    "*",
    [
    #(PF_IMAGE, "image", "Input image", None), #current image
    #(PF_DRAWABLE, "layer", "Input layer", None), #current layer
    (PF_STRING, "ok","Just OK to Continue", "OK"),
	
	],
    [],
    pointed_raster)

main()